#! /usr/bin/env python

import tomoesnml
import pandas as pd
import numpy as np
import os,sys
import random
import re
import psycopg2



def main(input_directory, output_path='./', output_file_name='bogus_transient'):
    input_directory_path  = input_directory
    psql_login = 'host=gwdata dbname=supernova user=tomoesn password='
    bogus_data = LoadBogusTransientData(input_directory_path)
    bogus_data.read_test_data()
    
    bogus_data.new_params.columns = ['new_number', 'new_x', 'new_y', 'new_ra', 'new_dec',
                                     'new_mag_auto', 'new_magerr_auto', 'new_a', 'new_b', 'new_theta',
                                     'new_elongation', 'new_fwhm', 'new_flux_auto', 'new_fluxerr_auto',
                                     'new_flux_max', 'new_class_star', 'new_ps_match', 'new_ps_type', 'new_ps_id',
                                     'new_obj_info_flag', 'new_ps_rmag', 'new_ps_dist']
    bogus_data.sub_params.columns = ['sub_number', 'sub_x', 'sub_y', 'sub_ra', 'sub_dec',
                                     'sub_mag_auto', 'sub_magerr_auto', 'sub_a', 'sub_b', 'sub_theta',
                                     'sub_elongation', 'sub_fwhm', 'sub_flux_auto', 'sub_fluxerr_auto',
                                     'sub_flux_max', 'sub_class_star', 'sub_value_zero', 'sub_saturation',
                                     'sub_mask', 'sub_edge']
    bogus_data.tom_params.columns = ['tom_a', 'tom_b', 'tom_bogus_tran_name', 'tom_date_obs', 'tom_dec', 'tom_det_id',
                                     'tom_elongation', 'tom_fits_name', 'tom_fits_path', 'tom_fwhm', 'tom_mag_auto',
                                     'tom_mag_auto_err', 'tom_magmp', 'tom_mjd', 'tom_project', 'tom_ra', 'tom_raw_id',
                                     'tom_source_id', 'tom_theta', 'tom_x', 'tom_y']

    with psycopg2.connect(psql_login) as connection:
        #with connection.cursor() as cur: cur.execute('SET statement_timeout TO 60000') # 60sec
        panstarrs_dataframe = get_panstarrs_data(connection, bogus_data.sub_params, max_num_of_inquire_objects=8000)
        
        # panstarrs_dataframe.columns: ['PS_MATCH','PS_TYPE','PS_ID','OBJ_INFO_FLAG','PS_RMAG','DIST_PS1']
        bogus_data.tom_params = bogus_data.tom_params.join(panstarrs_dataframe)
        connection.commit()

    params = pd.concat([bogus_data.tom_params, bogus_data.new_params, bogus_data.sub_params],axis=1)
    params.loc[(params['new_x'] == 0)&(params['new_y'] == 0)&(params['new_ra'] == 0)&(params['new_dec'] == 0),
               ['new_mag_auto','new_magerr_auto','new_a','new_b','new_theta','new_elongation','new_fwhm',
                'new_flux_auto','new_fluxerr_auto','new_flux_max','new_class_star','new_ps_rmag','new_ps_dist']] = np.nan
    params.loc[(params['new_x'] == 0)&(params['new_y'] == 0)&(params['new_ra'] == 0)&(params['new_dec'] == 0),['new_number']] = -1
    params.loc[(params['new_x'] == 0)&(params['new_y'] == 0)&(params['new_ra'] == 0)&(params['new_dec'] == 0),
               ['new_ps_match','new_ps_id','new_obj_info_flag']] = 0
    params.loc[(params['new_x'] == 0)&(params['new_y'] == 0)&(params['new_ra'] == 0)&(params['new_dec'] == 0),['new_ps_type']] = 'none'
    params.loc[(params['new_x'] == 0)&(params['new_y'] == 0)&(params['new_ra'] == 0)&(params['new_dec'] == 0),
               ['new_x','new_y','new_ra','new_dec']] = np.nan

    params.reset_index(inplace=True,drop=True)
    print 'Output Params : {}'.format(params.shape)
    print 'Output images : {}'.format(bogus_data.images.shape)

    if not os.path.isdir(output_path): os.makedirs(output_path)
    output_images_path = os.path.join(output_path, output_file_name+'.npy')
    output_params_path = os.path.join(output_path, output_file_name+'.csv')
    np.save(output_images_path, bogus_data.images)
    params.to_csv(output_params_path, index=False)



class LoadBogusTransientData(object):
    def __init__(self, _data_directory_path):
        self.data_directory_path = _data_directory_path
    
    @staticmethod
    def norm_image(ref_image_nd, new_image_nd, sub_image_nd):
        # Not use this function.
        ref_image_nd = ref_image_nd.reshape(ref_image_nd.shape[0],ref_image_nd.shape[1])
        new_image_nd = new_image_nd.reshape(new_image_nd.shape[0],new_image_nd.shape[1])
        sub_image_nd = sub_image_nd.reshape(sub_image_nd.shape[0],sub_image_nd.shape[1])

        ref_image_nd = (ref_image_nd-ref_image_nd.min())/(ref_image_nd.max()-ref_image_nd.min())
        new_image_nd = (new_image_nd-new_image_nd.min())/(new_image_nd.max()-new_image_nd.min())
        sub_image_nd = (sub_image_nd-sub_image_nd.min())/(sub_image_nd.max()-sub_image_nd.min())
        return [ref_image_nd, new_image_nd, sub_image_nd]

    @staticmethod
    def read_numpy_image_data(dir_path, raw_text):
        try:
            contents   = os.listdir(dir_path)
        except OSError:
            return None
        pattern        = re.compile(raw_text)
        npy_files_name = [os.path.join(dir_name, target_file_name)
                          for dir_name in contents 
                              for target_file_name in os.listdir(os.path.join(dir_path,dir_name))
                                  if os.path.isfile(os.path.join(dir_path, dir_name, target_file_name)) and pattern.match(target_file_name)
                         ]
        if len(npy_files_name)==0 :
            return None
        npy_files_name = sorted(npy_files_name)
        data_nd_list   = [np.load(os.path.join(dir_path, file_name)) for file_name in npy_files_name]
        #data_nd_list   = [np.stack(LoadRealTransientData.norm_image(*np.split(data_nd, 3, axis=2)), axis=-1)
        #                  for data_nd in data_nd_list]
        data_nd        = np.stack(data_nd_list, axis=0) 
        #print 'Size of nd array : {}'.format(data_nd.shape)
        return data_nd

    @staticmethod
    def read_csv_data(dir_path, raw_text):
        try:
            contents   = os.listdir(dir_path)
        except OSError:
            return None
        pattern        = re.compile(raw_text)
        csv_files_name = [os.path.join(dir_name, target_file_name)
                          for dir_name in contents 
                              for target_file_name in os.listdir(os.path.join(dir_path,dir_name)) 
                                  if os.path.isfile(os.path.join(dir_path, dir_name, target_file_name)) and pattern.match(target_file_name)]
        if len(csv_files_name)==0 :
            return None
        csv_files_name = sorted(csv_files_name)
        data_pd_list   = [pd.read_csv(os.path.join(dir_path, file_name)) for file_name in csv_files_name]

        #for file_name in csv_files_name: print os.path.join(dir_path, file_name)

        data_pd = pd.concat(data_pd_list, axis=0)
        #print 'Size of pd : {}'.format(data_pd.shape)
        return data_pd

    def read_test_data(self):

        input_directory_path = self.data_directory_path
        
        # -----------------------------------------------------------
        # Read data
        # -----------------------------------------------------------

        image_list       = []
        new_param_list   = []
        sub_param_list   = []
        tomoe_param_list = []

        try:
            contents   = os.listdir(input_directory_path)
        except OSError:
            return None
        
        for bogus_transient_name in contents:
            dir_path = os.path.join(input_directory_path, bogus_transient_name, 'analysis_data')
            #print 'LOAD : {}'.format(bogus_transient_name)
            search_text    = r'.*?psf.npy'
            image_nd       = LoadBogusTransientData.read_numpy_image_data(dir_path, search_text)

            search_text    = r'.*?tomoe_params.csv'
            tomoe_param_pd = LoadBogusTransientData.read_csv_data(dir_path, search_text)

            search_text    = r'.*?new_sex_params.csv'
            new_param_pd   = LoadBogusTransientData.read_csv_data(dir_path, search_text)

            search_text    = r'.*?sub_sex_params.csv'
            sub_param_pd   = LoadBogusTransientData.read_csv_data(dir_path, search_text)
        

            if ((new_param_pd is not None)   and (sub_param_pd is not None) and
                (tomoe_param_pd is not None) and (image_nd is not None)):
                image_list.append(image_nd)
                new_param_list.append(new_param_pd)
                sub_param_list.append(sub_param_pd)
                tomoe_param_list.append(tomoe_param_pd)

        image_nd       = np.vstack(image_list)
        new_param_pd   = pd.concat(new_param_list, axis=0)
        sub_param_pd   = pd.concat(sub_param_list, axis=0)
        tomoe_param_pd = pd.concat(tomoe_param_list, axis=0)

        print 'LOAD : Total'
        print 'image_nd    : {}'.format(image_nd.shape)
        print 'new_param   : {}'.format(new_param_pd.shape)
        print 'sub_param   : {}'.format(sub_param_pd.shape)
        print 'tomoe_param : {}'.format(tomoe_param_pd.shape)

        self.images       = image_nd
        self.new_params   = new_param_pd
        self.sub_params   = sub_param_pd
        self.tom_params   = tomoe_param_pd



def get_panstarrs_data(connection, dataframe, temp_table_name='hamasaki_table', max_num_of_inquire_objects=8000):

    def inquire_psql_database(ra_np, dec_np, temp_table_name):
        ra_dec_np       = np.stack([ra_np,dec_np], axis=-1)
        ra_dec_tuple    = tuple(map(tuple,ra_dec_np)) # ra_dec_tuple = ((ra1, dec1),(ra2, dec2)...(ra, dec))
        ra_dec_str      = ','.join(map(str, ra_dec_tuple))
        with connection.cursor() as cur:
            query = 'CREATE TEMPORARY TABLE {} (ra DOUBLE PRECISION, dec DOUBLE PRECISION)'.format(temp_table_name)
            cur.execute(query)

            # Insert (ra,dec)data to my_table
            query = 'INSERT INTO {} (ra,dec) VALUES {}'.format(temp_table_name, ra_dec_str)
            cur.execute(query)
        with connection.cursor('match_ps_table') as cur: # ---!!!CAUTION!!!--- Heavy Processing
            cur.itersize = 10000
            # Search ps1 catalog
            query = 'SELECT ss.* FROM {} AS t LEFT JOIN LATERAL (SELECT s."psId", s."objInfoFlag", s."rmag", q3c_dist(t.ra, t.dec, s.ra, s.dec) FROM ps_bright AS s WHERE q3c_join(t.ra, t.dec, s.ra, s.dec, 1./60) ORDER BY q3c_dist(t.ra, t.dec, s.ra, s.dec) ASC LIMIT 1 ) AS ss ON true'.format(temp_table_name)
            cur.execute(query)
            responses = cur.fetchall()
            data = [[0 if num is None else num for num in ps_param] for ps_param in responses]
        return data

    def get_ps_data(ps_info):
        match_rad   = 3./3600 # [arcsec]
        match       = False
        pstype      = None
        if ps_info:
            psId, objInfoFlag, rmag, dist_ps1 = ps_info
            pstype = 'star'

            if objInfoFlag:
                for iobj,objInfo in enumerate(['FEW','POOR','ICRF_QSO','HERN_QSO_P60','HERN_QSO_P05',
                                               'HERN_RRL_P60','HERN_RRL_P05','HERN_VARIABLE','TRANSIENT',
                                               'HAS_SOLSYS_DET','MOST_SOLSYS_DET','LARGE_PM','RAW_AVE',
                                               'FIT_AVE','FIT_PM','FIT_PAR','USE_AVE','USE_PM','USE_PAR',
                                               'NO_MEAN_ASTROM','STACK_FOR_MEAN','MEAN_FOR_STACK','BAD_PM',
                                               'EXT','EXT_ALT','GOOD','GOOD_ALT','GOOD_STACK','BEST_STACK',
                                               'SUSPECT_STACK','BAD_STACK']):
                    if objInfoFlag & 2**iobj:
                        if objInfo == 'EXT': pstype = 'extended'

        if dist_ps1:
            if dist_ps1 <= match_rad: match = True        

        return [match, pstype, psId, objInfoFlag, rmag, dist_ps1]

    ra_np  = dataframe['sub_ra'].values
    dec_np = dataframe['sub_dec'].values
    if ra_np.shape[0] > max_num_of_inquire_objects: raise Exception('Too many objects : {}'.format(ra_np.shape[0]))
    try: responses = inquire_psql_database(ra_np, dec_np, temp_table_name)
    except: raise
    else:
        ps_list = [get_ps_data(res) for res in responses]
        ps_df = pd.DataFrame(ps_list, columns=['ps_match','ps_type','ps_id','ps_obj_info_flag','ps_rmag','ps_dist'])
        return ps_df



if __name__=='__main__':
    print 'Start.'
    input_directory = '/gwkiso/hamasaki/test_data/bogus_transient'
    main(input_directory, output_path='./', output_file_name='bogus_transient')
    print 'Finish.'
    exit(0)
