#! /usr/bin/env python

import numpy as np
import pandas as pd
import os,sys
import re


def main(input_root_directory, output_directory='./'):

    #det_q_list   = [3]
    #det_num_list = [24,25]

    det_q_list   = [1,2,3,4]
    det_num_list = [11,12,13,14,15,16,
                    21,22,23,24,25,26,
                    31,32,33,34,35,
                    41,42,43,44]

    det_num_list = [q*100+num for q in det_q_list for num in det_num_list]

    artificial_real_images_list = []
    artificial_real_params_list = []
    artifact_images_list = []
    artifact_params_list = []

    for det_num in det_num_list:

        input_directory_path  = os.path.join(input_root_directory, 'rand_Q{}/analysis_data'.format(det_num))
        rand_all_data = read_learning_data(input_directory_path)

        input_directory_path  = os.path.join(input_root_directory, 'galx_Q{}/analysis_data'.format(det_num))
        galx_all_data = read_learning_data(input_directory_path)

        rand_artificial_real_images = rand_all_data[0]
        rand_artificial_real_params = rand_all_data[2]

        galx_artificial_real_images = galx_all_data[0]
        galx_artificial_real_params = galx_all_data[2]

        artifact_images = galx_all_data[1] # Use Only AROUND GALAXY Data
        artifact_params = galx_all_data[3] # Use Only AROUND GALAXY Data

        rand_artificial_real_params['PS_TYPE'] = rand_artificial_real_params.apply(check_ps_type, axis=1)
        galx_artificial_real_params['PS_TYPE'] = galx_artificial_real_params.apply(check_ps_type, axis=1)
        artifact_params['PS_TYPE']             = artifact_params.apply(check_ps_type, axis=1)

        rand_artificial_real_params.drop(columns=['Unnamed: 0'], inplace=True)
        galx_artificial_real_params.drop(columns=['Unnamed: 0'], inplace=True)
        artifact_params.drop(columns=['Unnamed: 0'], inplace=True)
        rand_artificial_real_params.drop(columns=['OBJ_INFO_FLAG'], inplace=True)
        galx_artificial_real_params.drop(columns=['OBJ_INFO_FLAG'], inplace=True)
        artifact_params.drop(columns=['OBJ_INFO_FLAG'], inplace=True)
        rand_artificial_real_params.drop(columns=['PS_STAR'], inplace=True)
        galx_artificial_real_params.drop(columns=['PS_STAR'], inplace=True)
        artifact_params.drop(columns=['PS_STAR'], inplace=True)

        old_columns = ['NUMBER', 'X_IMAGE', 'Y_IMAGE', 'ALPHA_J2000', 'DELTA_J2000',
                       'MAG_AUTO', 'MAGERR_AUTO', 'A_IMAGE', 'B_IMAGE', 'THETA_IMAGE',
                       'ELONGATION', 'FWHM_IMAGE', 'FLUX_AUTO', 'FLUXERR_AUTO', 'FLUX_MAX',
                       'CLASS_STAR', 'ARTIFICIAL_REAL', 'PS_TYPE', 'DIST_PS1', 'S2N', 'FITS_FILE_NAME',
                       'SQL_INFO_RAW_ID', 'SQL_INFO_FWHM', 'SQL_INFO_ZERO_MAG', 'SQL_INFO_LIM_MAG',
                       'TOO_CLOSE', 'VALUE_ZERO_REF', 'SATURATION', 'MASK', 'EDGE']

        new_columns = ['sub_number', 'sub_x', 'sub_y', 'sub_ra', 'sub_dec',
                       'sub_mag_auto', 'sub_magerr_auto', 'sub_a', 'sub_b', 'sub_theta',
                       'sub_elongation', 'sub_fwhm', 'sub_flux_auto', 'sub_fluxerr_auto', 'sub_flux_max',
                       'sub_class_star', 'artificial_real', 'ps_type', 'ps_dist', 'sub_snr', 'fits_name', 
                       'fits_raw_id', 'fits_fwhm', 'fits_zero_mag', 'fits_lim_mag',
                       'sub_too_close', 'sub_value_zero', 'sub_saturation', 'sub_mask', 'sub_edge']

        columns = {old_col: new_col for new_col, old_col in zip(new_columns, old_columns)}

        rand_artificial_real_params.rename(columns=columns, inplace=True)
        galx_artificial_real_params.rename(columns=columns, inplace=True)
        artifact_params.rename(columns=columns, inplace=True)

        column_name = 'fits_det_id'
        rand_artificial_real_params[column_name] = det_num
        galx_artificial_real_params[column_name] = det_num
        artifact_params[column_name]             = det_num

        column_name = 'object_type'
        rand_artificial_real_params[column_name] = 'rand_artificial_real'
        galx_artificial_real_params[column_name] = 'galx_artificial_real'
        artifact_params[column_name]             = 'artifact'

        rand_artificial_real_params.drop(columns=['artificial_real'], inplace=True)
        galx_artificial_real_params.drop(columns=['artificial_real'], inplace=True)
        artifact_params.drop(columns=['artificial_real'], inplace=True)

        artificial_real_images = np.concatenate([rand_artificial_real_images, galx_artificial_real_images], axis=0)
        artificial_real_params = pd.concat([rand_artificial_real_params, galx_artificial_real_params], axis=0)
        
        print 'DET{} : Artificial Real {}, Artifact {}'.format(det_num, artificial_real_params.shape[0], artifact_params.shape[0])
        
        artificial_real_images_list.append(artificial_real_images)
        artificial_real_params_list.append(artificial_real_params)
        artifact_images_list.append(artifact_images)
        artifact_params_list.append(artifact_params)
        

    artificial_real_images = np.concatenate(artificial_real_images_list, axis=0)
    artifact_images        = np.concatenate(artifact_images_list, axis=0)
    artificial_real_params = pd.concat(artificial_real_params_list, axis=0)
    artifact_params        = pd.concat(artifact_params_list, axis=0)
    
    if not os.path.isdir(output_directory): os.makedirs(output_directory)

    np.save(os.path.join(output_directory,'artificial_real.npy'), artificial_real_images)
    np.save(os.path.join(output_directory,'artifact.npy'), artifact_images)
    artificial_real_params.to_csv(os.path.join(output_directory,'artificial_real.csv'), index=False)
    artifact_params.to_csv(os.path.join(output_directory,'artifact.csv'), index=False)



def read_csv_data(dir_path, raw_text):
    lower_limit    = 0
    upper_limit    = 2500
    contents       = os.listdir(dir_path)
    pattern        = re.compile(raw_text)
    csv_files_name = [f for f in contents if os.path.isfile(os.path.join(dir_path, f)) and pattern.match(f)]
    # print 'Number of files : {}, Use file number between {} and {}'.format(len(csv_files_name),lower_limit,upper_limit)
    csv_files_name = sorted(csv_files_name)
    csv_files_name = csv_files_name[lower_limit:upper_limit]
    data_pd_list   = [pd.read_csv(os.path.join(dir_path, file_name))
                      for file_name in csv_files_name if pd.read_csv(os.path.join(dir_path, file_name)).shape[0] != 0 ]
    data_pd = pd.concat(data_pd_list, axis=0)
    #print 'Size of pd array : {}'.format(data_pd.shape)
    #print ''
    return data_pd

def read_numpy_image_data(dir_path, raw_text):
    lower_limit    = 0
    upper_limit    = 2500
    contents       = os.listdir(dir_path)
    pattern        = re.compile(raw_text)
    npy_files_name = [f for f in contents if os.path.isfile(os.path.join(dir_path, f)) and pattern.match(f)]
    # print 'Number of files : {}, Use file number between {} and {}'.format(len(npy_files_name),lower_limit,upper_limit)
    npy_files_name = sorted(npy_files_name)
    npy_files_name = npy_files_name[lower_limit:upper_limit]
    data_nd_list   = [np.load(os.path.join(dir_path, file_name))
                      for file_name in npy_files_name if np.load(os.path.join(dir_path, file_name)).shape[0] != 0 ]
    #data_nd_list   = [ np.array([ norm_image(img) for img in data_nd ]) for data_nd in data_nd_list ]
    data_nd        = np.concatenate(data_nd_list, axis=0)
    #print 'Size of nd array : {}'.format(data_nd.shape)
    #print ''
    return data_nd

def read_learning_data(input_directory_path):

    # -----------------------------------------------------------
    # Read data
    # -----------------------------------------------------------

    # print 'Read ref ArtificialReal image data'
    search_text                 = r'.*?_ref_ArtificialReal_psf.npy'
    ref_ArtificialReal_image_nd = read_numpy_image_data(input_directory_path, search_text)

    # print 'Read ref Artifact image data'
    search_text                 = r'.*?_ref_Artifact_psf.npy'
    ref_Artifact_image_nd       = read_numpy_image_data(input_directory_path, search_text)
    
    # print 'Read new ArtificialReal image data'
    search_text                 = r'.*?_new_ArtificialReal_psf.npy'
    new_ArtificialReal_image_nd = read_numpy_image_data(input_directory_path, search_text)

    # print 'Read new Artifact image data'
    search_text                 = r'.*?_new_Artifact_psf.npy'
    new_Artifact_image_nd       = read_numpy_image_data(input_directory_path, search_text)

    # print 'Read sub ArtificialReal image data'
    search_text                 = r'.*?_sub_ArtificialReal_psf.npy'
    sub_ArtificialReal_image_nd = read_numpy_image_data(input_directory_path, search_text)

    # print 'Read sub Artifact image data'
    search_text                 = r'.*?_sub_Artifact_psf.npy'
    sub_Artifact_image_nd       = read_numpy_image_data(input_directory_path, search_text)

    # print 'Read sub ArtificialReal param data'
    search_text                 = r'.*?_sub_ArtificialReal_param.csv'
    sub_ArtificialReal_param_pd = read_csv_data(input_directory_path, search_text)

    # print 'Read sub Artifact param data'
    search_text                 = r'.*?_sub_Artifact_param.csv'
    sub_Artifact_param_pd       = read_csv_data(input_directory_path, search_text)

    # -----------------------------------------------------------
    # Stack data
    # -----------------------------------------------------------
    
    ArtificialReal_images_nd = np.stack([ref_ArtificialReal_image_nd,
                                         new_ArtificialReal_image_nd,
                                         sub_ArtificialReal_image_nd], axis=3)
    Artifact_images_nd       = np.stack([ref_Artifact_image_nd,
                                         new_Artifact_image_nd,
                                         sub_Artifact_image_nd], axis=3)
    
    return ArtificialReal_images_nd, Artifact_images_nd, sub_ArtificialReal_param_pd, sub_Artifact_param_pd

def check_ps_type(pd_series):
    ps_type = 'none'
    if pd_series['PS_STAR'] == 1: ps_type = 'star'
    if pd_series['PS_STAR'] == 2: ps_type = 'extended'    
    return ps_type



if __name__=='__main__':
    print 'Start.'
    input_root_directory = '/gwkiso/hamasaki/dev/test9/learning_data'
    main(input_root_directory, output_directory='data')
    print 'Finish.'
    exit(0)
